local keql = LibStub("AceAddon-3.0"):GetAddon("keql")
local addon = keql
local Quixote = LibStub("LibQuixote-2.0")
local db, dbpc
local L = LibStub("AceLocale-3.0"):GetLocale("keql")


function keql:InitTracker()
	db, dbpc = keql.db.profile, keql.dbpc.profile
	MAX_QUESTWATCH_LINES = 50
	MAX_WATCHABLE_QUESTS = 10
	for i = 31, 50 do
		local string = QuestWatchFrame:CreateFontString("QuestWatchLine" .. i,"BACKGROUND","QuestWatchFontTemplate")
		string:SetPoint("TOPLEFT", getglobal("QuestWatchLine" .. (i-1)), "BOTTOMLEFT")
	end
	keql.TrackerQuests = {}
	self:RawHook("QuestWatch_Update", true)
	self:RawHook("RemoveQuestWatch",true)
	self:RawHook("IsQuestWatched", true)
	self:RawHook("GetNumQuestWatches", true)
	self:RawHook("AddQuestWatch", true)
	self:RawHook("GetQuestIndexForWatch", true)
	
	-- Hooking Secure Functions to prevent tainting
	hooksecurefunc("UIParent_ManageFramePositions",function()
		if dbpc.trackercorner and dbpc.trackerposx and dbpc.trackerposy then
			QuestWatchFrame:ClearAllPoints()
			QuestWatchFrame:SetPoint(dbpc.trackercorner,"UIParent","BOTTOMLEFT", dbpc.trackerposx, dbpc.trackerposy)
			--keql:AchievementFrameManagePosition()
		end
		if db.achievement.manage then
			-- manage the achievement tracker position
		end
	end)

    --self:RegisterEvent("Quixote_Quest_Gained")
	Quixote.RegisterCallback(keql, "Update", "Quixote_Update")
    --self:RegisterEvent("Quixote_Update")
    self:RegisterEvent("CHAT_MSG_SYSTEM")

	if dbpc.trackercorner and dbpc.trackerposx and dbpc.trackerposy then
		QuestWatchFrame:ClearAllPoints()
		QuestWatchFrame:SetPoint(dbpc.trackercorner,"UIParent","BOTTOMLEFT", dbpc.trackerposx, dbpc.trackerposy)
	else
		QuestWatchFrame:ClearAllPoints()
		QuestWatchFrame:SetPoint("TOPLEFT","UIParent","Center",0,0)
		keql:SavePositionTracker()
	end

	QuestWatchFrame:SetFrameStrata("BACKGROUND")
	QuestWatchFrame:SetMovable(true)
	QuestWatchFrame:EnableMouse(true)	
	QuestWatchFrame:SetScript("OnDragStart",function() if not dbpc.minimizedtracker then QuestWatchFrame.ismoving = true QuestWatchFrame:StartMoving() end end)
	QuestWatchFrame:SetScript("OnDragStop",function() QuestWatchFrame.ismoving = false QuestWatchFrame:StopMovingOrSizing() keql:SavePositionTracker() end)	
	
	keql.TrackerBackdrop = CreateFrame("Frame","QuestWatchFrameBackdrop",QuestWatchFrame)
	keql.TrackerBackdrop:ClearAllPoints()
	keql.TrackerBackdrop:SetPoint("TOPLEFT", QuestWatchFrame ,"TOPLEFT", -8, 0)
	keql.TrackerBackdrop:SetPoint("BOTTOMLEFT",QuestWatchFrame, "BOTTOMLEFT",-8,0)
	keql.TrackerBackdrop:SetPoint("TOPRIGHT", QuestWatchFrame ,"TOPRIGHT", 0, 0)	
	keql.TrackerBackdrop:SetPoint("BOTTOMRIGHT",QuestWatchFrame, "BOTTOMRIGHT", 0,0)	
	keql.TrackerBackdrop:SetFrameStrata("BACKGROUND")
	
	keql.TrackerBackdrop:SetBackdrop({
		bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = true, edgeSize = 16, tileSize = 16,
		insets = { left= 2, right = 2, top = 2, bottom = 2 }
		})
	keql.TrackerBackdrop:SetFrameLevel("1")
	keql.TrackerBackdrop:Show()	

	keql.TrackerTitleBar = CreateFrame("Frame","TrackerTitleBar", QuestWatchFrame)
	
	keql.TrackerTitleBar:ClearAllPoints()
	keql.TrackerTitleBar:SetHeight(25)
	keql.TrackerTitleBar:SetPoint("TOPLEFT",QuestWatchFrame, "TOPLEFT",-8,20)
	keql.TrackerTitleBar:SetPoint("TOPRIGHT",QuestWatchFrame, "TOPRIGHT",0,20)
	keql.TrackerTitleBar:SetBackdrop({
		bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = true, edgeSize = 16, tileSize = 16,
		insets = { left= 2, right = 2, top = 2, bottom = 2 }
		})			
	keql.TrackerTitleBar:SetFrameStrata("BACKGROUND")
	keql.TrackerTitleBar:SetFrameLevel("1")
	keql.TrackerTitleBar:SetAlpha(50)
	keql.TrackerTitleBar:SetMovable(true)
	keql.TrackerTitleBar:EnableMouse(true)
	keql.TrackerTitleBar:SetScript("OnDragStart",function() QuestWatchFrame:StartMoving() end)
	keql.TrackerTitleBar:SetScript("OnDragStop",function() QuestWatchFrame:StopMovingOrSizing() keql:SavePositionTracker() end)		
	keql.TrackerTitleBar:Show()
	
	keql.TrackerTitleBarText = keql.TrackerTitleBar:CreateFontString("TrackerTitleBarText","BACKGROUND","QuestWatchFontTemplate")
	keql.TrackerTitleBarText:SetPoint("LEFT",10,1)
	keql.TrackerTitleBarText:SetAlpha(1)

	keql.TrackerMinimizeButton = CreateFrame("Button","TrackerMinimizeButton", TrackerTitleBar)
	--keql.TrackerMinimizeButton:SetNormalTexture("Interface\\AddOns\\keql\\Images\\minimize_up")
	keql.TrackerMinimizeButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
	--keql.TrackerMinimizeButton:SetPushedTexture("Interface\\AddOns\\keql\\Images\\minimize_down")
	keql.TrackerMinimizeButton:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-Down")
	--keql.TrackerMinimizeButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	keql.TrackerMinimizeButton:SetHighlightTexture("Interface\\Buttons\\UI-MinusButton-Hilight")
	--keql.TrackerMinimizeButtonTex = keql.TrackerMinimizeButton:CreateTexture("Interface\\Buttons\\UI-MinusButton-Up")
	--keql.TrackerMinimizeButtonTex:SetAllPoints(keql.TrackerMinimizeButton)
	

	keql.TrackerMinimizeButton:SetWidth(16)
	keql.TrackerMinimizeButton:SetHeight(16)
	--keql.TrackerMinimizeButton:SetBackdrop({bgFile="Interface\\Buttons\\UI-MinusButton-Up", tile = false, edgeSize = 16, tileSize = 16, insets = { left= 0, right = 0, top = 0, bottom = 0 }})	
	keql.TrackerMinimizeButton:Show()
	keql.TrackerMinimizeButton:ClearAllPoints()
	keql.TrackerMinimizeButton:SetPoint("TOPRIGHT",TrackerTitleBar, "TOPRIGHT", -4,-4)
	keql.TrackerMinimizeButton:SetScript("OnClick", function() 
		if dbpc.minimizedtracker then 
			keql:TrackerMaximize() 
		else 
			keql:TrackerMinimize() 
		end 
	end)
	
	QuestWatchFrame:SetAlpha(db.tracker.alpha)
	if db.tracker.addnewquests then
		AUTO_QUEST_WATCH = "1"
	else
		AUTO_QUEST_WATCH = "0"
	end
	if not db.tracker.lock then
		QuestWatchFrame:RegisterForDrag("LeftButton")
		keql.TrackerTitleBar:RegisterForDrag("LeftButton")
	end
	keql:CheckMouseEvents()
	QuestWatch_Update()
end


function keql:SavePositionTracker()
	keql:TrackerLockCornerForGrowth()
end

function keql:TrackerMinimize()
	dbpc.minimizedtracker = true
	--keql.TrackerMinimizeButtonTex:SetTexture("Interface\\Buttons\\UI-PlusButton-Up")
	keql.TrackerMinimizeButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up")
	keql.TrackerMinimizeButton:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-Down")	
	for i = 1, 50 do
		local CurFrame = getglobal("QuestWatchLine"..(i))
		CurFrame:Hide()
	end	
	local minWidth = keql.TrackerTitleBarText:GetWidth()+32
	QuestWatchFrame:SetHeight(32)
	QuestWatchFrame:SetWidth(minWidth)
	keql.TrackerBackdrop:Hide()
end

function keql:TrackerMaximize()
	dbpc.minimizedtracker = false
	--keql.TrackerMinimizeButtonTex:SetTexture("Interface\\Buttons\\UI-MinusButton-Up")
	keql.TrackerMinimizeButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
	keql.TrackerMinimizeButton:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-Down")	
	for i = 1, 50 do
		local CurFrame = getglobal("QuestWatchLine"..(i))
		CurFrame:Show()
	end	
	keql.TrackerBackdrop:Show()
	QuestWatch_Update()
end

--
-- Events
--

function keql:Quixote_Update()
	QuestWatch_Update()
end

function keql:CHAT_MSG_SYSTEM(arg1, arg2)
	--self:Print("CHAT_MSG_SYSTEM:"..arg2)
	if db.tracker.addnewquests then
		--self:Print("db.tracker.addnewquests:"..tostring(db.tracker.addnewquests))
		if string.find(arg2, L["keql_QUEST_ACCEPTED"].." .+") then
			local temp = string.gsub(arg2, L["keql_QUEST_ACCEPTED"].." ", "")
			keql.TrackerQueue = temp
			--self:Print(temp)
			keql:ManageQuests()
		end
	end
end

--
-- Hooks
--

function keql:AddQuestWatch(questIndex)
	
	local questName, level = GetQuestLogTitle(questIndex)
	local questLogHeader, tempId

	if IsQuestWatched(questIndex) then
		return
	else
		isHeader = false
		tempId = questIndex
		while (not isHeader) do
			questLogHeader, _, _, _, isHeader = GetQuestLogTitle(tempId)
			tempId = tempId-1
		end

		table.insert(dbpc.tracked, questLogHeader..","..level..","..questName)
	end
	keql:SortWatchedQuests()
end

function keql:RemoveQuestWatch(questIndex)
	if questIndex <= 0 then
		return
	end
	local questName, level = GetQuestLogTitle(questIndex)

	local questLogHeader, isHeader, tempId

	isHeader = false
	tempId = questIndex
	while (not isHeader) do
		questLogHeader, _, _, _, isHeader = GetQuestLogTitle(tempId)
		tempId = tempId-1
	end

	local temp = questLogHeader..","..level..","..questName

	if table.getn(dbpc.tracked) > 0 then
		for i=1, table.getn(dbpc.tracked) do
			if dbpc.tracked[i] == temp then
				table.remove(dbpc.tracked , i)
				break
			end
		end
	end
	keql:SortWatchedQuests()
end

function keql:IsQuestWatched(questIndex)
	
	local questName, level = self.hooks.GetQuestLogTitle(questIndex)
	local questLogHeader, isHeader, tempId
	local _

	isHeader = false
	tempId = questIndex
	while (not isHeader and tempId > 0) do
		questLogHeader, _, _, _, isHeader = self.hooks.GetQuestLogTitle(tempId)
		tempId = tempId-1
	end
	

	local temp = questLogHeader..","..level..","..questName
	--local temp = Quixote:GetQuestById(questIndex)
	if table.getn(dbpc.tracked) > 0 then
		for i=1, table.getn(dbpc.tracked) do
			if dbpc.tracked[i] == temp then
				return true
			end
		end
	end
	return false
end


-- Returns the number of watched quests
function keql:GetNumQuestWatches()
	return table.getn(dbpc.tracked)
end

function keql:GetQuestIndexForWatch(id)
	local numEntries, numQuests = GetNumQuestLogEntries()
	local questLogTitleText, level
	local questLogHeader, isHeader, tempId
	local temp, currentHeader=nil
	local _

	for i=1, numEntries, 1 do
		questLogTitleText, level, _, _, isHeader, _ = GetQuestLogTitle(i)
		if isHeader then
			currentHeader = questLogTitleText
		else
			temp = currentHeader..","..level..","..questLogTitleText
			if  temp == dbpc.tracked[id]  then
				return i
			end
		end
	end
	return 0
end
function addon:QuestWatch_Update()
	local questIndex, isRemoved, curZone, numObjectives, objText
	local completedQuests, questWatchMaxWidth, watchTextID = 0, 0, 1
	local Color = {}
	addon:launcherUpdate()
	dbpc.TrackerQuests = {}
	if not Quixote.firstDone then
		return
	end
	keql:ManageQuests()
	--for _,uid in pairs(dbpc.tracked) do
	for i=1, GetNumQuestWatches() do
		questIndex = GetQuestIndexForWatch(i)
		--self:Debug("uid:"..tostring(uid))
		qUID, qID = nil, nil
		if questIndex then 
			qUID,qID, qTitle, qLvl, qTag, qObj, qComplete, qGroup, qDaily, qZone = Quixote:GetQuestById(questIndex)
		end
		if qUID then
			isRemoved = false
			if db.tracker.removecompleted and qComplete == 1 then
				isRemoved = true
				completedQuests = completedQuests +1
			end	
			
			if (curZone == nil or curZone ~= qZone) and not isRemoved then
				curZone = qZone
				if db.tracker.showzones then
					tempwidth = keql:AddWatchLineText(watchTextID,curZone,db.tracker.fontheight,db.colors.ZoneColor, db.tracker.fontheight+2)
					if tempwidth > questWatchMaxWidth then questWatchMaxWidth = tempwidth end
					watchTextID = watchTextID + 1
				end
			end
			if qID > 0 and not isRemoved then
				--self:Debug("QuestWatch_Update qUID:"..tostring(qUID))
				-- Set Quest Title Color
				if qComplete == 1 then
					--Completed
					completedQuests = completedQuests +1
					Color = db.colors.QuestColorCompleted
				elseif qComplete == -1 then
					Color = db.colors.QuestColorFailed
				else
					-- Normal
					if db.colors.QuestColorLevels then
						Color = GetDifficultyColor(qLvl)
					else
						Color = db.colors.QuestColor
					end
				end
				--self:Debug("db.general.ShowLevels:"..tostring(db.general.ShowLevels))
				if db.questlog.showlevels then
					--self:Debug("db.questlog.showlevels:"..tostring(db.questlog.showlevels))
					tempwidth = keql:AddWatchLineText(watchTextID,"  ".."["..qLvl..self:GetQuestTag(qTag, qDaily, qGroup).."] "..qTitle,db.tracker.fontheight, Color)
				else
					tempwidth = keql:AddWatchLineText(watchTextID,"  "..qTitle,db.tracker.fontheight, Color)				
				end
				dbpc.TrackerQuests[watchTextID] = qUID
				if tempwidth > questWatchMaxWidth then questWatchMaxWidth = tempwidth end
				watchTextID = watchTextID + 1
				
				numObjectives = qObj
				if db.tracker.hidecompletedobj and qComplete == 1 then
					numObjectives = 0
				end				
				if db.tracker.hidequestcompletedobj and qComplete == 1 then
					numObjectives = 0
				end
				markerID = 0
				if  numObjectives > 0 then
					-- Iterate through Objectives
					for objective, got, need, t in Quixote:IterateObjectivesForQuest(qUID) do
						--self:Debug("IterateObjectivesForQuest: objective:"..tostring(objective).." got:"..tostring(got).." need:"..tostring(need).." t:"..tostring(t))
						ObjColor = db.colors.ObjectiveColor
						ObjColorComplete = db.colors.ObjectiveColorCompleted
						local oComplete = (got == need) or false
						if oComplete then
							Color = ObjColorComplete
						else								
							Color = ObjColor
						end
						objText = "   - "
						objText = objText..objective
						if t == "item" or t == "monster" or t == "reputation" then
							objText = objText..": "..got.."/"..need
						end
						if (oComplete and not db.tracker.hidecompletedobj) or (not oComplete) then
							tempwidth = keql:AddWatchLineText(watchTextID,objText,db.tracker.FontHeight,Color)
							if tempwidth > questWatchMaxWidth then questWatchMaxWidth = tempwidth end
							watchTextID = watchTextID + 1
						end
					end
				end
			end
		end
	end	
	-- Show Tracker header if set, set quest count in tracker header
	if db.tracker.trackerheader then
		t1, _, t2 = QuestWatchLine1:GetFont()
		keql.TrackerTitleBarText:SetFont(t1, db.tracker.fontheight+2, t2)
		keql.TrackerTitleBarText:SetHeight(db.tracker.fontheight+2)	
		keql.TrackerTitleBarText:Show()
		local questnums = GetNumQuestWatches()
		local _,questEntries = GetNumQuestLogEntries()
		if db.tracker.removecompleted then
			keql.TrackerTitleBarText:SetText(L["Quest Tracker"].. " ("..questnums - completedQuests.."/"..questEntries..")")		
		else
			keql.TrackerTitleBarText:SetText(L["Quest Tracker"].. " ("..questnums.."/"..questEntries..")")				
		end
		keql.TrackerMinimizeButton:Show()
		keql.TrackerTitleBar:Show()
	else
		keql.TrackerTitleBar:Hide()
		keql.TrackerMinimizeButton:Hide()
		if dbpc.minimized then
			keql:TrackerMaximize()
		end		
	end
	-- Tracker Background handling
	keql.TrackerBackdrop:SetBackdropColor(db.colors.TrackerBGColor.r, db.colors.TrackerBGColor.g, db.colors.TrackerBGColor.b, db.colors.TrackerBGColor.a)
	keql.TrackerBackdrop:SetBackdropBorderColor(db.colors.TrackerBorderColor.r, db.colors.TrackerBorderColor.g, db.colors.TrackerBorderColor.b, db.colors.TrackerBorderColor.a)
	keql.TrackerTitleBar:SetBackdropColor(db.colors.TrackerBGColor.r, db.colors.TrackerBGColor.g, db.colors.TrackerBGColor.b, db.colors.TrackerBGColor.a)
	keql.TrackerTitleBar:SetBackdropBorderColor(db.colors.TrackerBorderColor.r, db.colors.TrackerBorderColor.g, db.colors.TrackerBorderColor.b, db.colors.TrackerBorderColor.a)		

	--Hide unused Watchlines
	for i=watchTextID, MAX_QUESTWATCH_LINES do
		getglobal("QuestWatchLine"..i):Hide()
	end
	
	--Hide the tracker if nothing is watched or Show it and resize it
	if ( watchTextID == 1 ) then
		--Set the tracker indicator
		QuestLogTrackTracking:SetVertexColor(1.0, 0, 0)
		QuestWatchFrame:Hide()
		return
	else
		-- Set the tracker indicator
		QuestLogTrackTracking:SetVertexColor(0, 1.0, 0)
		QuestWatchFrame:Show()
		QuestWatchFrame:SetHeight((watchTextID * db.tracker.fontheight+1) + (watchTextID * 4) + 12)

		QuestWatchFrame:SetWidth(questWatchMaxWidth + 12)
		-- If the tracker is smaller than the minwidht, resize it
		local minWidth = keql.TrackerTitleBarText:GetWidth()+32
		local curWidth = QuestWatchFrame:GetWidth()
		if curWidth < minWidth then
			QuestWatchFrame:SetWidth(minWidth)
		end
		--self:Debug("questWatchMaxWidth:"..tostring(questWatchMaxWidth))
	end
	--Position the Tracker
	UIParent_ManageFramePositions()
	--Lock Tracker Corner for resizing
	keql:TrackerLockCornerForGrowth()
		
	--Minimize the Tracker if its state is set to minimized
	if dbpc.minimizedtracker then
		keql:TrackerMinimize()
	end
end

-- For adding Text to Watcher
function keql:AddWatchLineText(id,text,size,color,height)
	local curWidth, tempWidth
	local _
	--local linenr
	if size == nil then size = db.tracker.fontheight end
	if color == nil then color = {r=1,g=1,b=1} end
	if height == nil then height = db.tracker.fontheight end
	watchText = getglobal("QuestWatchLine"..id)
	if watchText ~= nil then
		t1, _, t2 = watchText:GetFont()
		--watchText:SetWidth(nil)
		watchText:SetHeight(0)
		watchText:SetText(text)
		watchText:SetNonSpaceWrap(false)
		watchText:SetFont(t1, size, t2)
		tempWidth = watchText:GetWidth()
		--curWidth = watchText:GetStringWidth()

		watchText:SetHeight(height)
		--watchText:SetWidth(curWidth)

		if ( id > 1 ) then
			watchText:SetPoint("TOPLEFT", "QuestWatchLine"..(id - 1), "BOTTOMLEFT", 0, -4)
		end
		watchText:SetTextColor(color.r, color.g, color.b)
		watchText:Show()	
	end
	return tempWidth
end

function keql:TrackerLockCornerForGrowth()
	local Left = QuestWatchFrame:GetLeft()
	local Right = QuestWatchFrame:GetRight()
	local Top = QuestWatchFrame:GetTop()
	local Bottom = QuestWatchFrame:GetBottom()
	local lock
	local pointone
	local pointtwo
	local TOPBOTTOM_MEDIAN = 384
	local LEFTRIGHT_MEDIAN = 512
	if Left and Right and Top and Bottom then
		if Bottom < TOPBOTTOM_MEDIAN and Top > TOPBOTTOM_MEDIAN then
			local topcross = Top - TOPBOTTOM_MEDIAN
			local bottomcross = TOPBOTTOM_MEDIAN - Bottom
			if bottomcross > topcross then
				lock = "BOTTOM"
				pointtwo = Bottom
			else
				lock = "TOP"
				pointtwo = Top
			end
		elseif Top > TOPBOTTOM_MEDIAN then
			lock = "TOP"
			pointtwo = Top
		elseif Bottom < TOPBOTTOM_MEDIAN then
			lock = "BOTTOM"
			pointtwo = Bottom
		end
		if Left < LEFTRIGHT_MEDIAN and Right > LEFTRIGHT_MEDIAN then
			local leftcross = LEFTRIGHT_MEDIAN - Left
			local rightcross = Right - LEFTRIGHT_MEDIAN
			if rightcross > leftcross then
				lock = lock.."RIGHT"
				pointone = Right
			else
				lock = lock.."LEFT"
				pointone = Left
			end
		elseif Left < LEFTRIGHT_MEDIAN then
			lock = lock.."LEFT"
			pointone = Left
		elseif Right > LEFTRIGHT_MEDIAN then
			lock = lock.."RIGHT"
			pointone = Right
		end
		if lock and lock ~= "" and pointone and pointtwo then
			QuestWatchFrame:ClearAllPoints()
			QuestWatchFrame:SetPoint(lock,"UIParent","BOTTOMLEFT",pointone,pointtwo)
			dbpc.trackercorner = lock
			dbpc.trackerposx = pointone
			dbpc.trackerposy = pointtwo
		elseif dbpc.trackercorner and dbpc.trackerposx and dbpc.trackerposy then
			QuestWatchFrame:ClearAllPoints()
			QuestWatchFrame:SetPoint(dbpc.trackercorner,"UIParent","BOTTOMLEFT",dbpc.trackerposx,dbpc.trackerposy)
		end
	end
end
